import * as assert from 'assert';
import * as mongoose from "mongoose";

const currencies_schema: mongoose.Schema = new mongoose.Schema({ // USA dollar:
    common_name: String, // Dollar
    common_symbol: {type: String}, // $
    description: {type: String, default: ""},
    iso_code: {type: Number}, // 840
    iso_symbol: String, // USD
    material_icon: {type: String, default: ""},
    substitute: {type: mongoose.Schema.Types.Mixed},
    substitution_date: {type: Date, default: Date.now}
});
// mongoose automatically looks for the plural, lowercase version of the model name...
// So, 'currencies' collection is created within MongoDB:
const currencies_model = mongoose.model('currencies', currencies_schema);
const Dollar = new currencies_model({
    common_name: "Dollar",
    common_symbol: "$",
    description: "First leading currency in the world...",
    iso_code: 840,
    iso_symbol: "USD",
    material_icon: "attach_money",
    substitute: null
});
console.assert(Dollar.get('common_symbol') === "$"); // Common style...
const {message} = new assert.AssertionError({
    actual: Dollar.get('common_symbol'),
    expected: "$",
    operator: "strictEqual"
});
try {
    assert.strictEqual(Dollar.get('common_symbol'), "$");
} catch (error: unknown) {
    assert(error instanceof assert.AssertionError);
    assert.strictEqual(error.message, message);
    assert.strictEqual(error.name, 'AssertionError');
    assert.strictEqual(error.actual, Dollar.get('common_symbol'));
    assert.strictEqual(error.expected, "$");
    assert.strictEqual(error.code, 'ERR_ASSERTION');
    assert.strictEqual(error.operator, 'strictEqual');
    assert.strictEqual(error.generatedMessage, true);
}

// Database within MongoDB Atlas:
const Database_URI = "mongodb+srv://FB:Sauvagnon64@web-scraping-98xi5.mongodb.net/Web_scraping?retryWrites=true&w=majority";
mongoose.connect(Database_URI).then((value: mongoose.Mongoose) => {
    console.log("Connected database: '" + (value.connections.length > 0 ? value.connections[0].name : "?")
        + "' on '" + (value.connections.length > 0 ? value.connections[0].host : "?") + "'");
});
const connection: mongoose.Connection = mongoose.connection;
connection.on('error', console.error.bind(console, 'connection error:'));
connection.once('open', () => { // Remove existing data (for test):
    currencies_model.deleteMany({common_symbol: "$"});
    Dollar.save().then(() => { // Then, check if present:
        currencies_model.find({common_symbol: "$"}).exec().then(currency => {
                console.log(currency);
                connection.close().then(() => {
                    console.log("\nSee you later!");
                });
            }
        );
    });
});


